package blosum;


public class ProteinMatrix extends Matrix {
	
	/* 
	 *numCouple is the number of couple of amino acids without gaps
	 */
	 
	private int numCouple = 0;
	private int identity;
	
	/**
	 * Method ProteinMatrix construct a Matrix containing the occurrences
	 * of the couples of amino acids in the 2 sequences.
	 *
	 * @param s1 first Sequence of amino acids
	 * @param s2 second Sequence of amino acids.
	 *
	 */
	public ProteinMatrix(Sequence s1, Sequence s2) throws Exception{
		super();
		String str1 = s1.getSequence();
		String str2 = s2.getSequence();
		int i,j;
		int max = Math.min(str1.length(), str2.length());
  	identity = 0;
		for (int k = 0; k < max; k++){
			i = AminoAcid.indexOfAmin(str1.charAt(k));
			j = AminoAcid.indexOfAmin(str2.charAt(k));
			//check if both characters are different from gaps
			if (i > 0 && j > 0){
				numCouple++;
				addCouple(i,j);
			}
			if (i == j && i > 0){
				identity++;
			}
		}
  	identity = (identity * 100) / max;
	}

	 /**
	 * Method addCouple: add 1 to the occurences of amino acids (i, j) and (j, i)
	 *
	 * @param i 
	 * @param j 
	 *
	 */
	private void addCouple(int i, int j){
		setValue(i-1,j-1,(this.getValue(i-1,j-1) + 1));
	}
	
	/**
	 * Method getFrequence
	 *
	 * @param x char representing an amino acid
	 * @param y char representing an amino acid
	 *
	 * @return the occurences of the couple (x,y) in 2 sequences.
	 *
	 */
	public float getFrequence(char x, char y) throws Exception{
		int i = AminoAcid.indexOfAmin(x);
		int j = AminoAcid.indexOfAmin(y);
		return getValue(i-1,j-1)/this.numCouple;
	}

  /**
	 * Method getFrequence
	 *
	 * @param i index of an amino acid
	 * @param j index of an amino acid
	 *
	 * @return the occurences of the couple (i,j) in 2 sequences.
	 *
	 */
	public float getFrequence(int i, int j) {
		return getValue(i-1,j-1)/this.numCouple;
	}

	/**
	 * Method getIdentity
	 *
	 *
	 * @return the percentage identity of two sequences 
	 *
	 */
	public int getIdentity() {
		return identity;
	}
}
