package blosum.applet;

import blosum.*;
import blosum.utils.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

/**
* @(#)BlosumApplet.java
*
* Applet application that splits the Blosum Score into numbers of biological significance
*
* @author Pussini Marco, Marica Rigo
*/
public class BlosumApplet extends JApplet {
	
	// Be sure to set the Default LookAndFeel
	static {
		try {
	    	UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
	    } catch (Exception ex) {}
	}
	
	/* 
	 * Component hierarchy aproximated (for the components listed below)
	 *
	 * 	BlosumApplet's contentPanel
	 *		|
	 *		+ mainPanel (JPanel)
	 *			|
	 *			+ inputPanel (JPanel)
	 *			|	|
	 *			|	+ chooserPanel (JPanel)
	 *			|	|	|
	 *			|	|	+ loader1 (SequenceLoader) [one file mode] [certificate accepted]
	 *			|	|	+ loader2a (SequenceLoader) [two files mode]
	 *			|	|	- loader2b (SequenceLoader) [two files mode]
	 *			|	|
	 *			|	- choicesPanel (JPanel) [certificate accepted]
	 *			|		|
	 *			|		+ twoFile (JRadioButton)
	 *			|		- oneFile (JRadioButton)
	 *			|
	 *			+ seqPanel (JPanel)
	 *			|	|
	 *			|	+ name1 (JLabel)
	 *			|	- name2 (JLabel)
	 *			|
	 *			- buttPanel (JPanel)
	 *				|
	 *				+ compute (JButton)
	 *				- blomat (JComboBox)
	 */
	private JPanel inputPanel = this.createTitledPanel("Input", new BorderLayout(5,5));
	private JPanel chooserPanel = new JPanel(new CardLayout());
	private JPanel choicesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
	private JRadioButton oneFile = new JRadioButton("Single File"); // For choosing the single file
	private JRadioButton twoFiles = new JRadioButton("Two Files"); // For choosing the two files
	private JPanel seqPanel = this.createTitledPanel("Sequences:", new GridLayout(2,1));
	private JPanel mainPanel;
	private JPanel buttPanel = this.createTitledPanel("", new BorderLayout());
	private JButton compute = new JButton("Compute");
	private JLabel name1 = new JLabel(" ");
	private JLabel name2 = new JLabel(" ");
	private SequenceLoader loader1;
	private SequenceLoader loader2a;
	private SequenceLoader loader2b;
	private JComboBox blomat= new JComboBox();
	
	private int[] matrices = {35, 40, 50, 62, 80, 100}; // The blosum matrices' indexes
	private Sequence[] sequences = new Sequence[2];
	private Color back = Color.WHITE; // background color
	private Color fore = Color.BLACK; // foreground color
	
	public void init() {
		this.readParametres(); // reads the parametres from the <APPLET> tag
		
		this.getContentPane().setLayout(new FlowLayout(FlowLayout.CENTER));
		this.getContentPane().setBackground(this.back);
		this.getContentPane().setForeground(this.fore);
		mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.PAGE_AXIS));
		mainPanel.setOpaque(false);
		// InputPanel definition =========================================================
		JLabel[] labels2a = new JLabel[1];
		JLabel[] labels2b = new JLabel[1];
		labels2a[0] = name1;
		labels2b[0] = name2;
		if (SequenceLoader.canUseFiles()) { // checks if the certificate has been accepted
			JLabel[] labels1 = new JLabel[2];
			// Prepare the name labels and the loaders for the two sequences
			labels1[0] = name1;
			labels1[1] = name2;
			loader1 = new SequenceLoader("Sequences url",2);
			// we listen for the sequence loading ("one file" mode)
			loader1.addSequenceLoaderListener(new SequenceLoaderListener() {
				public void sequenceChoosed(SequenceLoader source) {
					setSequences(source);
				}
			});
			loader2a = new SequenceLoader("First sequence's file url",1);
			loader2b = new SequenceLoader("Second sequence's file url",1);
			// Adds the two loaders for the "two files" mode
			JPanel p2 = new JPanel(new GridLayout(2,1));
			p2.setOpaque(false);
			p2.add(loader2a);
			p2.add(loader2b);
			chooserPanel.add(p2,twoFiles.getText());
			// Adds the single loader for the "one file" mode
			JPanel p1 = new JPanel(new BorderLayout());
			p1.setOpaque(false);
			p1.add(loader1, BorderLayout.NORTH);
			chooserPanel.add(p1,oneFile.getText());
			chooserPanel.setOpaque(false);
			// Prepares the radio buttons
			oneFile.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					changePanel((JRadioButton)ae.getSource());
				}
			});
			twoFiles.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					changePanel((JRadioButton)ae.getSource());
				}
			});
			oneFile.setSelected(false);
			oneFile.setOpaque(false);
			twoFiles.setSelected(true);
			twoFiles.setOpaque(false);
			ButtonGroup bg = new ButtonGroup();
			bg.add(oneFile);
			bg.add(twoFiles);
			choicesPanel.add(twoFiles);
			choicesPanel.add(oneFile);
			choicesPanel.setOpaque(false);
			// Adds the ChecBoxes
			inputPanel.add(choicesPanel, BorderLayout.NORTH);
		} else {
			// The certificate has not been accepted so the sequences have to be inserted manually
			// Prepares the two loaders (one for each sequence)
			loader2a = new SequenceLoader("First sequence",1);
			loader2b = new SequenceLoader("Second sequence",1);
			JPanel p3 = new JPanel(new GridLayout(2,1));
			p3.setOpaque(false);
			p3.add(loader2a);
			p3.add(loader2b);
			chooserPanel.add(p3,"");
			chooserPanel.setOpaque(false);
		}
		// As for the loader1, we listen for the loading of the sequences from the other two loaders
		loader2a.addSequenceLoaderListener(new SequenceLoaderListener() {
			public void sequenceChoosed(SequenceLoader source) {
				setSequences(source);
			}
		});
		loader2b.addSequenceLoaderListener(new SequenceLoaderListener() {
			public void sequenceChoosed(SequenceLoader source) {
				setSequences(source);
			}
		});
		// Finally adds the inputPanel to the mainPanel
		inputPanel.add(chooserPanel, BorderLayout.CENTER);
		inputPanel.setOpaque(false);
		mainPanel.add(inputPanel);
		mainPanel.add(Box.createRigidArea(new Dimension(5,5)));
		// seqPanel definition =========================================================
		name1.setHorizontalAlignment(JLabel.LEFT);
		name2.setHorizontalAlignment(JLabel.LEFT);
		seqPanel.add(name1);
		seqPanel.add(name2);
		seqPanel.setOpaque(false);
		mainPanel.add(seqPanel);
		mainPanel.add(Box.createRigidArea(new Dimension(5,5)));
		// buttPanel definition =========================================================
		// fills the JComboBox
		for (int i = 0; i < this.matrices.length; i++) {
			this.blomat.insertItemAt("BLOSUM "+matrices[i], i);
		}
		// BLOSUM 35 selected as default
		this.blomat.setSelectedIndex(0);
		JPanel p4 = new JPanel(new FlowLayout());
		p4.setOpaque(false);
		p4.add(new JLabel("Blosum matrix:", JLabel.RIGHT));
		p4.add(blomat);
		buttPanel.add(p4, BorderLayout.WEST);
		compute.setEnabled(false);
		buttPanel.add(compute, BorderLayout.EAST);
		buttPanel.setOpaque(false);
		compute.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				compute();
			}
		});
		mainPanel.add(buttPanel);
		mainPanel.add(Box.createRigidArea(new Dimension(5,5)));
	
		this.getContentPane().add(mainPanel);
		this.validate(); // A good resizing of all the recently added components
	}
	
	/**
     * Reads the parametres from the <APPLET> tag.
     * In this last version the only parametres accepted are BACKCOLOR and FORECOLOR
     */
	private void readParametres() {
		try {
			this.back = new Color(Integer.parseInt(this.getParameter("BACKCOLOR"), 16));
			this.fore = new Color(Integer.parseInt(this.getParameter("FORECOLOR"), 16));
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
     * Invoked from a SequenceLoader, sets the two input sequences and their names
     *
     * @param source The SequenceLoader that fires the event
     */
	private void setSequences(SequenceLoader source) {
		Sequence[] seqs = source.getSequences(); // A sequenceLoader may have readed more than one sequence!
		if (source.equals(loader1)) {
			// If the source is loader1 we are sure that we have two sequences
			this.sequences[0] = seqs[0];
			this.name1.setText(seqs[0].getName());
			this.sequences[1] = seqs[1];
			this.name2.setText(seqs[1].getName());
		} else if (source.equals(loader2a)) { // In the other cases we have only one sequence
			this.sequences[0] = seqs[0];
			this.name1.setText(seqs[0].getName());
		} else {
			this.sequences[1] = seqs[0];
			this.name2.setText(seqs[0].getName());
		}
		// We don't want to permit a bad resize of the Labels due to long sequences names
		if (this.name1.WIDTH > this.WIDTH - 20) {
			int h = this.name2.getPreferredSize().height;
			this.name1.setSize(this.WIDTH-20, h);
			this.name1.setPreferredSize(new Dimension(this.WIDTH-20, h));
		}
		if (this.name2.WIDTH > this.WIDTH - 20) {
			int h = this.name2.getPreferredSize().height;
			this.name2.setSize(this.WIDTH-20, h);
			this.name2.setPreferredSize(new Dimension(this.WIDTH-20, h));
		}
		this.validate();
		// enables the "compute" button if it's necessary
		this.compute.setEnabled(this.sequences[0] != null && this.sequences[1] != null);
	}
	
	/**
	 * Creates a JPanel with a titled border with a specific title and a specific LayoutManager
	 *
	 * @param title The string to be placed as title
	 * @param layout The LayoutManager assigned to the JPanel
	 *
	 * @return A JPanel with a titled border with a specific title and a specific LayoutManager
	 */
	public static JPanel createTitledPanel(String title, LayoutManager layout) {
		JPanel panel = new JPanel(layout);
		panel.setBorder(BorderFactory.createTitledBorder(title));
	return panel;
	}
	
	/**
	 * Invoked from the "compute" button, starts the computation
	 */
	private void compute() {
		int mat = this.matrices[this.blomat.getSelectedIndex()];
		Computation c = new Computation(this, this.sequences[0], this.sequences[1],
			mat, this.getCodeBase());
		// We are interested to the end of the computation
		c.addComputationListener(new ComputationListener() {
			public void computationTerminated(Computation.Results res) {
				terminated(res);
			}
		});
		// Disable the ui until the computation ends
		this.setEnabled(false);
		c.startComputation(); // starts the computation
	}
	
	/**
	 * Called from the current computation
	 *
	 * @param res The result of the computation
	 */
	private void terminated(Computation.Results res) {
		this.setEnabled(true); // enables the ui
		if (res != null) {
			// If the computation succeded, the results are shown
			OutputFrame out = new OutputFrame(res, this.getCodeBase());
		}
	}
	
	/**
	 * Called from the JRadioButtons, It swetches the Panels containig the loaders
	 *
	 * @param source The JRadioButton that fires the event
	 */
	private void changePanel(JRadioButton source) {
		((CardLayout)this.chooserPanel.getLayout()).show(this.chooserPanel,source.getText());
	}

}
