package blosum.applet;

import blosum.*;
import blosum.utils.*;
import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.EventListener;
import java.net.URL;

public class OutputFrame extends JFrame{

  private JPanel main = new JPanel(new BorderLayout(5,5));
  private JButton save = new JButton("Save HTML");
  private JScrollPane jsp;
  private JTextPane jep = new JTextPane();
  private JFileChooser chooser;
  private Sequence s1, s2, subseq;
  private String cmpstr;
  private int idx, perc_identity, mat, score;
  private float sxy, mutuainfo, divxy, divx, divy;
  private URL codeBase;


	/**
	* Method OutputFrame creates a JFrame in order to show the output of the computation.
	*
	* @param res is the result of the computation.
	* @param base is URL specifying the codebase.
	*
	*/	
	
  public OutputFrame(Computation.Results res, URL base) {
    super("Output");
    this.codeBase = base;
    this.idx = res.OFFSET;
    this.sxy = res.NBS;
    this.perc_identity = res.IDENTITY;
    this.mat = res.MAT;
    this.mutuainfo = res.IXY;
    this.divxy = res.DXY;
    this.divx = res.DX;
    this.divy = res.DY;
    this.score = res.SCORE;
    this.s1 = res.FIRSTSEQ;
    this.s2 = res.SECONDSEQ;
    this.cmpstr = res.COMPARESTR;
    try {
      this.subseq = this.s1.subsequence(this.idx, this.s2.getSequence().length());
    } catch(Exception e) {
      this.subseq = s2;
    }
    this.getContentPane().setLayout(new BorderLayout());
    main.setBorder(BorderFactory.createEmptyBorder(10,10,10,5));
    jep.setContentType("text/html");
	jep.setEditable(false);
	JScrollPane jsp = new JScrollPane(jep);
    main.add(jsp, BorderLayout.CENTER);
    JPanel p = new JPanel(new FlowLayout(FlowLayout.RIGHT));
    p.add(save);
    save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveRequest();
			}
		});
    main.add(p, BorderLayout.SOUTH);
	this.getContentPane().add(main, BorderLayout.CENTER);
    jep.setText(this.getOutputText());
    this.pack();
    this.setSize(800,720);
    this.setVisible(true);
  }

	/**
	 * Method getOutputText 
	 * 
	 * @return a String containing the output of the computation in html format.
	 */
	private String getOutputText() {
		int n = 20;
	    String str1 = s1.getName();
	    String str2 = s2.getName();
	    String subStr = subseq.getName();
	    String out = "";
	    String images = this.codeBase.toString() + "/images/";
		out += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
		out += "<html>\n";
		out += "<body bgcolor=\"#b0c4de\">\n";
	
		/************************************/
	    out += "<p align=right>Software developed by the Bioinformatics Lab<br>Department of  Mathematics and Computer Science,<br>\n";
	    out += "University of Udine (M.Pussini, M.Rigo)</p>\n";
	    out += "<table borderColor=#ffffff width=\"72%\" align=center bgColor=#ffffff border=0>\n";
	    out += "<tbody>\n";
	    out += "<tr borderColor=#ffffff bgColor=#ffffff>\n";
	    out += "<td width=\"18%\"><img height=147 src=\"" + images + "proteina.jpg\"\n";
	    out += "width=100></td>\n";
	    out += "<td width=\"82%\">\n";
	    out += "<h1 align=center>&nbsp;</h1>\n";
	    out += "<h1 align=center>Splitting the BLOSUM score into numbers of biological significance</h1>\n";
	    out += "<h5 align=right>F. Fabris, A. Sgarro and A. Tossi<br>Universit degli studi di Trieste</h5></td></tr>\n";
	    out += "</tbody></table>\n";
		out += "<p>&nbsp;</p>\n";
	
		/************************************/
	
		out += "<strong>"+ str1 + " (" + s1.getSequence().length() + " bps)</strong>\n";
		out += "<font size = h3 face = \"courier\"><pre>\n";
	    int c;
	    for (c = 70; c < s1.getSequence().length(); c+=70)
	            out += s1.getSequence().substring(c-70, c)+"\n";
	    out += s1.getSequence().substring(c-70)+"\n";
	    out += "</pre></font>\n";
	    out += "<strong>" + str2 + " (" + s2.getSequence().length() + " bps)</strong>\n";
	    out += "<font size = h3 face = \"courier\"><pre>\n";
	    for (c = 70; c < s2.getSequence().length(); c+=70)
	            out += s2.getSequence().substring(c-70, c)+"\n";
	    out += s2.getSequence().substring(c-70)+"\n";        
	    out += "</pre></font>\n";
	    out +=  "<strong>Blosum "+mat+"</strong>\n";
		out += "<hr>\n";		
	    out += "<br>\n";
	
	
		/************************************/
	
		out += "<p><strong>" + str1 + "</strong>\n";
		out += "<font size = h3 face = \"courier\"><pre>" + subseq.getSequence()+"\n";
		out +=  cmpstr +"\n";
		out += s2.getSequence() + "</pre></font>\n";
		out += "<strong>" +str2 + "</strong><br>\n";
		out += "<strong>Position:</strong>\n";
		out +=  (idx+1) +"</p>\n";
	
	
		/************************************/
	
		out += "<p>&nbsp;</p>\n";
		out += "<table width=\"90%\" border=\"1\" align=\"center\" bgcolor=\"#FFFFFF\">\n";
		out += "<tbody>\n";
		out += "<tr bgcolor=\"#FFFFE6\">\n";
		out += "<td width=\"12%\">\n";
		out += "<div align=center><font size=h2>Normalized score*</font></div></td>\n";
		out += "<td width=\"12%\">\n";
		out += "<div align=center><font size=h2>Stochastic correlation</font></div></td>\n";
		out += "<td width=\"18%\">\n";
		out += "<div align=center><font size=h2>Distance of the substitutions from the protein model</font></div></td>\n";
		out += "<td width=\"10%\">\n";
		out += "<div align=center><font size=h2>Protein typicality of the 1st seq.</font></div></td>\n";
		out += "<td width=\"10%\">\n";
		out += "<div align=center><font size=h2>Protein typicality of the 2nd seq.</font></div></td>\n";
		out += "<td width=\"8%\">\n";
		out += "<div align=center><font size=h2>BLOSUM Score</font></div></td>\n";
		out += "<td width=\"8%\">\n";
		out += "<div align=center><font size=h2>Percentage identity</font></div></td>\n";
		out += "<td width=\"12%\">\n";
		out += "<div align=center><font size=h2>BLOSUM spectrum</font></div></td></tr>\n";		
		out += "<tr>\n";
		out += "<td width=\"12%\">\n";
		out += "<div align=center>NBS*</div></td>\n";
		out += "<td width=\"12%\">\n";
		out += "<div align=center>I(X;Y)</div></td>\n";
		out += "<td width=\"18%\">\n";
		out += "<div align=center>D(Fxy//Pab)</div></td>\n";
		out += "<td width=\"10%\">\n";
		out += "<div align=center>D(Fx//P)</div></td>\n";
		out += "<td width=\"10%\">\n";
		out += "<div align=center>D(Fy//P)</div></td>\n";
		out += "<td width=\"8%\">\n";
		out += "<div align=center>BLOSUM Score</div></td>\n";
		out += "<td width=\"8%\">\n";
		out += "<div align=center>Identity %</div></td>\n";
		out += "<td width=\"12%\" ROWSPAN=2 VALIGN=CENTER ALIGN=CENTER>\n";
	
		out += "<TABLE ALIGN=CENTER VALIGN=CENTER BORDER=0 CELLSPACING=0 CELLPADDING=0>\n";
		out += "<TR>\n";
		if (sxy > 0) {
			out += "<TD VALIGN=BOTTOM ALIGN=CENTER><IMG SRC=\"" + images + "blu.jpg\" width=8 height=";
			out += Math.round(sxy * n) + " hspace=4></TD>\n";
		}else
			out += "<TD WIDTH=16/>";
		out += "<TD VALIGN=BOTTOM ALIGN=CENTER><IMG SRC=\"" + images + "nero.jpg\" width=8 height=";
		out += Math.round(mutuainfo * n) + " hspace=4></TD>\n";
		out += "<TD WIDTH=16/>\n";
		out += "<TD VALIGN=BOTTOM ALIGN=CENTER><IMG SRC=\"" + images + "nero.jpg\" width=8 height=";
		out += Math.round(divx * n) + " hspace=4 ></TD>\n";
		out += "<TD VALIGN=BOTTOM ALIGN=CENTER><IMG SRC=\"" + images + "nero.jpg\" width=8 height=";
		out += Math.round(divy * n) + " hspace=4></TD>\n";
		out += "</TR>\n";
		out += "<TR><TD COLSPAN=5 ALIGN=CENTER><IMG SRC=\"" + images + "nero.jpg\" width=80 height=1></TD></TR>\n";
		out += "<TR>\n";
		if (sxy < 0) {
			out += "<TD VALIGN=TOP ALIGN=CENTER><IMG SRC=\"" + images + "rosso.jpg\" width=8 height=";
			out +=  Math.round(-sxy * n) + " hspace=4></TD>\n";
		}else
			out += "<TD WIDTH=16/>\n";
		out += "<TD WIDTH=16/>\n";
		out += "<TD VALIGN=TOP ALIGN=CENTER><IMG SRC=\"" + images + "nero.jpg\" width=8 height=";
		out += Math.round(divxy * n) + " hspace=4></TD>\n";
		out += "<TD WIDTH=16/>\n";
		out += "<TD WIDTH=16/>\n";
		out += "</TR>\n";
		out += "</tbody>\n";
		out += "</TABLE>\n";
	
		out += "</TD>\n";
		out += "</tr>\n";
		out += "<tr>\n";
		out += "<td><DIV ALIGN = CENTER>" + sxy + "</DIV></td>\n";
		out += "<td><DIV ALIGN = CENTER>" + mutuainfo + "</DIV></td>\n";
		out += "<td><DIV ALIGN = CENTER>" + divxy + "</DIV></td>\n";
		out += "<td><DIV ALIGN = CENTER>" + divx + "</DIV></td>\n";
		out += "<td><DIV ALIGN = CENTER>" + divy + "</DIV></td>\n";
		out += "<td><DIV ALIGN = CENTER>" + score + "</DIV></td>\n";
		out += "<td><DIV ALIGN = CENTER>" + perc_identity + "</DIV></td>\n";
		out += "</tr>\n";
		out += "</table>\n";
	    out += "<h4 align=center>NBS* = I(X,Y) - D(Fxy//Pab) + D(Fx//P) + D(Fy//P)</h4>\n";
	    out += "</html>\n";
	    return out;
	}

	/**
	 * Method saveOutputhtml save an html file with the output of the computation
	 *
	 * @param outdir is the directory in which it will be saved the output.
	 *
	 */
	public void saveOutputhtml(File outdir){
        String str1 = s1.getName();
        String str2 = s2.getName();
        String out = "";
		String filename;
		if((str1.compareTo("")==0 && str2.compareTo("")==0) || 
		     	(s1.getCode().compareTo("")==0 && s2.getCode().compareTo("")==0))
		      	filename = mat + "_Result";
		else 	        
		      	filename =  mat + "_" + s1.getCode() + "_" + s2.getCode();	    
	        try{
		    	File directory = new File(outdir.getAbsolutePath()+File.separator+filename);
		    	if (directory.exists()) {
		    		int answer = JOptionPane.showConfirmDialog(null, directory +
						"\n alredy exists. Do you want to replace it?", "Overwrite?",
						JOptionPane.YES_NO_OPTION);
					if (answer == 0) {
						deleteFiles(directory);
				    } else return;
		    	} else if(!directory.mkdir()) {
		    		throw new IOException("Cannot create the directory "+directory);
		    	}
		    	/*Copy protein.jpg into directory*/
				URL imgIN = new URL(this.codeBase + "/images/proteina.jpg");
				File img1OUT = new File(directory.getAbsolutePath()+File.separator+"proteina.jpg");
				InputUtils.copy(imgIN,img1OUT);
				
		    	/*Copy istogram.jpg into directory*/
				SpectrumIstogram si = new SpectrumIstogram(mutuainfo,divxy,divx,divy);
				File file = new File(directory.getAbsolutePath()+"/istogram.png");
				if(! si.write(file)) throw new IOException("Impossible write istogram.png");
	
				out+="<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
				out+="<html>\n";
				out+="<head>\n";
				out+="<title>Result</title>\n";
				out+="<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n";
				out+="</head>\n";
				out+="<body bgcolor=\"#b0c4de\">\n";
	
				/************************************/
	
				out+="<p align=right>Software developed by the Bioinformatics Lab<br>Department of  Mathematics and Computer Science,<br>\n";
		        out+="University of Udine (M. Pussini, M.Rigo)</p>\n";
		        out+="<table borderColor=#ffffff width=\"72%\" align=center bgColor=#ffffff border=0>\n";
		        out+="<tbody>\n";
		        out+="<tr borderColor=#ffffff bgColor=#ffffff>\n";
		        out+="<td width=\"18%\"><img height=147 src=\"proteina.jpg\"width=100></td>\n";
		        out+="<td width=\"82%\">\n";
		        out+="<h1 align=center>&nbsp;</h1>\n";
		        out+="<h1 align=center>Splitting the BLOSUM score into numbers of biological significance</h1>\n";
		        out+="<h5 align=right>F. Fabris, A. Sgarro and A. Tossi<br>Universit degli studi di Trieste</h5></td></tr>\n";
		        out+="</tbody></table>\n";
				out+="<p>&nbsp;</p>\n";
				
				/************************************/
							
		        int c;
				out+="<strong>"+ str1 + " (" + s1.getSequence().length() + " bps)</strong>\n";
				out+="<font size = h3 face = \"courier\"><pre>\n";
				for (c = 70; c < s1.getSequence().length(); c+=70)
					out+=s1.getSequence().substring(c-70, c)+"\n";
				out+=s1.getSequence().substring(c-70)+"\n";
				out+="</pre></font>\n";
				out+="<strong>" + str2 + " (" + s2.getSequence().length() + " bps)</strong><br>\n";
				out+="<font size = h3 face = \"courier\"><pre>\n";
				for (c = 70; c < s2.getSequence().length(); c+=70)
					out+=s2.getSequence().substring(c-70, c)+"\n";
				out+=s2.getSequence().substring(c-70)+"\n";	    
				out+="</pre></font>\n";
				out+="<strong>Blosum "+mat+"</strong>\n";
				out+="<hr>\n";
				out+="<br>\n";
				
				/************************************/
				
				out+="<p><strong>" + str1 + "</strong><br>\n";
				out+="<font size = h3 face = \"courier\"><pre>" + subseq.getSequence()+"\n";
				out+=cmpstr+"\n";
				out+=s2.getSequence() + "</pre></font>"+"\n";
				out+="<strong>" +str2 + "</strong><br>"+"\n";
				out+="<strong>Position:</strong>"+"\n";
				out+=(idx+1) +"</p>\n";
				
				/************************************/	
				   
		        out+="<p>&nbsp;</p>\n";
				out+="<table width=\"80%\" border=\"1\" align=\"center\" bgcolor=\"#FFFFFF\">\n";
				out+="<tbody>\n";
				out+="<tr bgcolor=\"#FFFFE6\">\n";
				out+="<td width=\"12%\">\n";
				out+="<div align=center><font size=h2>Normalized score*</font></div></td>\n";
				out+="<td width=\"12%\">\n";
				out+="<div align=center><font size=h2>Stochastic correlation</font></div></td>\n";
				out+="<td width=\"18%\">\n";
				out+="<div align=center><font size=h2>Distance of the substitutions from the protein model</font></div></td>\n";
				out+="<td width=\"10%\">\n";
				out+="<div align=center><font size=h2>Protein typicality of the 1st seq.</font></div></td>\n";
				out+="<td width=\"10%\">\n";
				out+="<div align=center><font size=h2>Protein typicality of the 2nd seq.</font></div></td>\n";
				out+="<td width=\"8%\">\n";
				out+="<div align=center><font size=h2>BLOSUM Score</font></div></td>\n";
				out+="<td width=\"8%\">\n";
				out+="<div align=center><font size=h2>Percentage identity</font></div></td>\n";
				out += "<td width=\"12%\">\n";
				out+="<div align=center><font size=h2>BLOSUM spectrum</font></div></td></tr>\n";
				
				out+="<tr>\n";
				out+="<td width=\"12%\">\n";
				out+="<div align=center>NBS*</div></td>\n";
				out+="<td width=\"12%\">\n";
				out+="<div align=center>I(X;Y)</div></td>\n";
				out+="<td width=\"18%\">\n";
				out+="<div align=center>D(Fxy//Pab)</div></td>\n";
				out+="<td width=\"10%\">\n";
				out+="<div align=center>D(Fx//P)</div></td>\n";
				out+="<td width=\"10%\">\n";
				out+="<div align=center>D(Fy//P)</div></td>\n";
				out+="<td width=\"8%\">\n";
				out+="<div align=center>BLOSUM Score</div></td>\n";
				out+="<td width=\"8%\">\n";
				out+="<div align=center>Identity %</div></td>\n";
				out+="<td width=\"12\" ROWSPAN=2 VALIGN=BOTTOM ALIGN=CENTER>\n";		
				out+="<img src=\"istogram.png\">\n";
				out+="</TD>\n";
				out+="</tr>\n";
				out+="<tr>\n";
				out+="<td><DIV ALIGN = CENTER>" + sxy + "</DIV></td>\n";
				out+="<td><DIV ALIGN = CENTER>" + mutuainfo + "</DIV></td>\n";
				out+="<td><DIV ALIGN = CENTER>" + divxy + "</DIV></td>\n";
				out+="<td><DIV ALIGN = CENTER>" + divx + "</DIV></td>\n";
				out+="<td><DIV ALIGN = CENTER>" + divy + "</DIV></td>\n";
				out+="<td><DIV ALIGN = CENTER>" + score + "</DIV></td>\n";
				out+="<td><DIV ALIGN = CENTER>" + perc_identity + "</DIV></td>\n";
				out+="</tr>\n";
				out+="</tbody>\n";
				out+="</table>\n";
		        out+="<h4 align=center>NBS* = I(X,Y) - D(Fxy//Pab) + D(Fx//P) + D(Fy//P)</h4>\n";
				out+="</html>\n";
				/************************************/					
				/*Copy _.html into directory*/
				FileWriter outfile = new FileWriter(new File(directory.getAbsolutePath()+File.separator+filename+".html"), false);
		        outfile.write(out);
		        outfile.close();
				JOptionPane.showMessageDialog(null, "Result saved successfully in \n"+ directory.getAbsolutePath(), "", JOptionPane.INFORMATION_MESSAGE);
			}catch(IOException ioe){
			   	JOptionPane.showMessageDialog(null, ioe.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
	}
	
	/**
	 * Method saveRequest allows to choose a directory in which to save the output.
	 *
	 *
	 */
	 
	private void saveRequest() {
		try {
	    	if (chooser == null) {
				chooser = new JFileChooser();
	    		chooser.setDialogTitle("Select output directory");
			}
	    	chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	       }catch (Exception ex) {
		        JOptionPane.showMessageDialog(null,
		        "<html>In order to save the result into your file system<br>you have to accept the certificate!</html>",
		        "Warning", JOptionPane.WARNING_MESSAGE);
			    	chooser = null;
	    	}
			if (chooser != null && chooser.showDialog(null,"Select") == JFileChooser.APPROVE_OPTION)
		    	saveOutputhtml(chooser.getSelectedFile());
	}
	
	/**
	 * Method deleteFiles deletes the files containing in the specified directory.
	 *
	 * @param dir 
	 */
	private void deleteFiles(File dir) {
		File img = new File(dir.getAbsolutePath()+File.separator+"proteina.jpg");
		File ist = new File(dir.getAbsolutePath()+"/istogram.png");
		if (img.exists()) {
			img.delete();
	    }
	    if (ist.exists()) {
			ist.delete();
	    }
	}
}