package blosum.utils;

import java.io.*;
import java.net.*;
import java.util.Vector;


public class InputUtils {
	
	/**
	 * Method loadFile: Opens and reads the specified file and stores
	 * each readed line into the given Vector
	 *
	 * @param filename: Name of the file to be read
	 * @param data: Vector where store data
	 *
	 */
	public static void loadFile(String filename, Vector data) throws Exception{
		File file = new File(filename);
		BufferedReader br;
		try {
			 br = new BufferedReader(new FileReader(file));
		} catch (FileNotFoundException fnfe) {
			throw new Exception("File \""+filename+"\" does not exists.");
		}
		if (data == null) {
			data = new Vector();
		}
		String readed;
		try {
			while ((readed = br.readLine()) != null) {
		    	data.add(readed); // The string may be ""
		    }
	    } catch (IOException ioe) {
	    	throw new Exception("Error while loading from \""+filename+"\".");
	    }
	    try {
	    	br.close();
	    } catch (Exception ex) {}
	}
	
	/**
	 * Method loadURL: Opens a connection to the given URL and reads each line of the file
	 * storing it into the given Vector
	 *
	 * @param url: the URL to connect to
	 * @param data: Vector where store data
	 *
	 */
	public static void loadURL(URL url, Vector data) throws Exception{
		BufferedReader br;
		try {
			 br = new BufferedReader(new InputStreamReader(url.openStream()));
		} catch (IOException ioe) {
			throw new Exception("Cannot connect to \""+url+"\".");
		}
		if (data == null) {
			data = new Vector();
		}
		String readed;
		try {
			while ((readed = br.readLine()) != null) {
				if (readed.length()>0) {
		    		data.add(readed);
			    }
		    }
	    } catch (IOException ioe) {
	    	throw new Exception("Error while loading from \""+url+"\".");
	    }
	    try {
	    	br.close();
	    } catch (Exception ex) {}
	}

	/**
	 * Method copy: Makes a copy of the file specified by the URL in
	 *
	 *
	 * @param in: the URL to connect to
	 * @param out: Vector where store data
	 *
	 */
	 
	 public static void copy(URL in, File out) {
	  try {
		  InputStream fr = in.openStream();
		  FileOutputStream fw = new FileOutputStream(out);
		  byte[] buf = new byte[1024];
		  int readed;
		  while((readed = fr.read(buf)) > 0) {
		    fw.write(buf,0,readed);
	  	  }
		  fw.flush();
		  fw.close();
		  fr.close();
		  } catch(Exception ex) {ex.printStackTrace();}
	  }
	
}